@@ -1,12 +1,7 @@
-import logging
-import re
-import time
-import urllib
-
+# -*- coding: utf-8 -*-
 from AccessControl import ClassSecurityInfo
 from AccessControl.PermissionRole import rolesForPermissionOn
-from AccessControl.Permissions import (
-    manage_zcatalog_entries as ManageZCatalogEntries)
+from AccessControl.Permissions import manage_zcatalog_entries as ManageZCatalogEntries  # noqa
 from AccessControl.Permissions import search_zcatalog as SearchZCatalog
 from Acquisition import aq_base
 from Acquisition import aq_inner
@@ -16,29 +11,38 @@ from App.special_dtml import DTMLFile
 from BTrees.Length import Length
 from DateTime import DateTime
 from OFS.interfaces import IOrderedContainer
+from plone.i18n.normalizer.base import mapUnicode
+from plone.indexer import indexer
+from plone.indexer.interfaces import IIndexableObject
 from Products.CMFCore.CatalogTool import CatalogTool as BaseTool
 from Products.CMFCore.CatalogTool import _mergedLocalRoles
+from Products.CMFCore.indexing import processQueue
 from Products.CMFCore.permissions import AccessInactivePortalContent
 from Products.CMFCore.utils import _checkPermission
 from Products.CMFCore.utils import _getAuthenticatedUser
 from Products.CMFCore.utils import getToolByName
 from Products.CMFPlone import DISCUSSION_ANNOTATION_KEY
-from Products.CMFPlone.PloneBaseTool import PloneBaseTool
 from Products.CMFPlone.interfaces import INonStructuralFolder
 from Products.CMFPlone.interfaces import IPloneCatalogTool
+from Products.CMFPlone.PloneBaseTool import PloneBaseTool
 from Products.CMFPlone.utils import base_hasattr
 from Products.CMFPlone.utils import safe_callable
 from Products.CMFPlone.utils import safe_unicode
 from Products.ZCatalog.ZCatalog import ZCatalog
-from plone.i18n.normalizer.base import mapUnicode
-from plone.indexer import indexer
-from plone.indexer.interfaces import IIndexableObject
+from zExceptions import Unauthorized
 from zope.annotation.interfaces import IAnnotations
 from zope.component import queryMultiAdapter
-from zope.interface import Interface
+from zope.component.hooks import getSite
 from zope.interface import implementer
+from zope.interface import Interface
 from zope.interface import providedBy
 
+import logging
+import re
+import six
+import time
+import urllib
+
 
 logger = logging.getLogger('Plone')
 
@@ -124,6 +128,18 @@ BLACKLISTED_INTERFACES = frozenset((
 ))
 
 
+def catalog_get_all(catalog, unique_idx='UID'):
+    """Get all brains from the catalog.
+    """
+    res = [
+        catalog({
+            unique_idx: catalog._catalog.getIndexDataForRID(it)[unique_idx]
+        })[0]
+        for it in catalog._catalog.data
+    ]
+    return res
+
+
 @indexer(Interface)
 def allowedRolesAndUsers(obj):
     """Return a list of roles and users with View permission.
@@ -162,6 +178,7 @@ def object_provides(obj):
 def zero_fill(matchobj):
     return matchobj.group().zfill(4)
 
+
 num_sort_regex = re.compile('\d+')
 
 
@@ -198,6 +215,7 @@ def getObjPositionInParent(obj):
         return ordered.getObjectPosition(obj.getId())
     return 0
 
+
 SIZE_CONST = {'KB': 1024, 'MB': 1024 * 1024, 'GB': 1024 * 1024 * 1024}
 SIZE_ORDER = ('GB', 'MB', 'KB')
 
@@ -270,11 +288,64 @@ def getIcon(obj):
     return bool(getattr(obj.aq_base, 'image', False))
 
 
+@indexer(Interface)
+def mime_type(obj):
+    return aq_base(obj).getPrimaryField().getContentType(obj)
+
+
 @indexer(Interface)
 def location(obj):
     return obj.getField('location').get(obj)
 
 
+class ContentIndexer(object):
+    """An instance of this class can be passed to ZopeFindAndApply
+    """
+
+    def __init__(self, catalog, queue_interval=100, log_progress=100, idxs=None):
+        self.catalog = catalog
+        self.queue_interval = queue_interval
+        self.log_progress = log_progress
+        self.counter = 0
+        self.idxs = idxs
+
+    def index_discussion(self, obj):
+        # index conversions from plone.app.discussion
+        annotations = IAnnotations(obj)
+        if DISCUSSION_ANNOTATION_KEY not in annotations:
+            return
+        conversation = annotations[DISCUSSION_ANNOTATION_KEY]
+        conversation = conversation.__of__(obj)
+        for comment in conversation.getComments():
+            try:
+                self.catalog.indexObject(comment)
+            except StopIteration:  # pragma: no cover
+                pass
+            self.counter += 1
+
+    def __call__(self, obj, path):
+        if not base_hasattr(obj, 'reindexObject'):
+            return
+        if not safe_callable(obj.reindexObject):
+            return
+        self.counter += 1
+        try:
+            obj.reindexObject(idxs=self.idxs)
+            self.index_discussion(obj)
+        except TypeError:
+            # Catalogs have 'reindexObject' as well, but they
+            # take different args, and will fail
+            pass
+        except Exception:
+            logger.exception(
+                'Problem indexing {0}'.format(obj.absolute_url())
+            )
+        if self.queue_interval and not(self.counter % self.queue_interval):
+            processQueue()
+        if self.log_progress and not(self.counter % self.log_progress):
+            logger.info('Indexed {0} objects.'.format(self.counter))
+
+
 @implementer(IPloneCatalogTool)
 class CatalogTool(PloneBaseTool, BaseTool):
     """Plone's catalog tool"""
@@ -304,7 +375,7 @@ class CatalogTool(PloneBaseTool, BaseTool):
         # Safe removal of an index.
         try:
             self.manage_delIndex(index)
-        except:
+        except Exception:
             pass
 
     def _listAllowedRolesAndUsers(self, user):
@@ -362,8 +433,57 @@ class CatalogTool(PloneBaseTool, BaseTool):
 
     @security.private
     def getCounter(self):
+        processQueue()
         return self._counter is not None and self._counter() or 0
 
+    @security.private
+    def allow_inactive(self, query_kw):
+        """Check, if the user is allowed to see inactive content.
+        First, check if the user is allowed to see inactive content site-wide.
+        Second, if there is a 'path' key in the query, check if the user is
+        allowed to see inactive content for these paths.
+        Conservative check: as soon as one path is disallowed, return False.
+        If a path cannot be traversed, ignore it.
+        """
+        allow_inactive = _checkPermission(AccessInactivePortalContent, self)
+        if allow_inactive:
+            return True
+
+        paths = query_kw.get('path', False)
+        if not paths:
+            return False
+
+        if isinstance(paths, dict):
+            # Like: {'path': {'depth': 0, 'query': ['/Plone/events/']}}
+            # Or: {'path': {'depth': 0, 'query': '/Plone/events/'}}
+            paths = paths.get('query', [])
+
+        if isinstance(paths, basestring):
+            paths = [paths]
+
+        objs = []
+        site = getSite()
+        for path in list(paths):
+            path = path.encode('utf-8')  # paths must not be unicode
+            try:
+                site_path = '/'.join(site.getPhysicalPath())
+                parts = path[len(site_path) + 1:].split('/')
+                parent = site.unrestrictedTraverse('/'.join(parts[:-1]))
+                objs.append(parent.restrictedTraverse(parts[-1]))
+            except (KeyError, AttributeError, Unauthorized):
+                # When no object is found don't raise an error
+                pass
+
+        if not objs:
+            return False
+
+        allow = True
+        for ob in objs:
+            allow = allow and\
+                _checkPermission(AccessInactivePortalContent, ob)
+
+        return allow
+
     @security.protected(SearchZCatalog)
     def searchResults(self, REQUEST=None, **kw):
         # Calls ZCatalog.searchResults with extra arguments that
@@ -375,6 +495,9 @@ class CatalogTool(PloneBaseTool, BaseTool):
         # effectiveRange checking entirely even for those without portal
         # wide AccessInactivePortalContent permission.
 
+        # Make sure any pending index tasks have been processed
+        processQueue()
+
         kw = kw.copy()
         show_inactive = kw.get('show_inactive', False)
         if isinstance(REQUEST, dict) and not show_inactive:
@@ -383,18 +506,35 @@ class CatalogTool(PloneBaseTool, BaseTool):
         user = _getAuthenticatedUser(self)
         kw['allowedRolesAndUsers'] = self._listAllowedRolesAndUsers(user)
 
-        if not show_inactive \
-           and not _checkPermission(AccessInactivePortalContent, self):
+        if not show_inactive and not self.allow_inactive(kw):
             kw['effectiveRange'] = DateTime()
 
+        # filter out invalid sort_on indexes
+        sort_on = kw.get('sort_on') or []
+        if isinstance(sort_on, six.string_types):
+            sort_on = [sort_on]
+        valid_indexes = self.indexes()
+        try:
+            sort_on = [idx for idx in sort_on if idx in valid_indexes]
+        except TypeError:
+            # sort_on is not iterable
+            sort_on = []
+        if not sort_on:
+            kw.pop('sort_on', None)
+        else:
+            kw['sort_on'] = sort_on
+
         return ZCatalog.searchResults(self, REQUEST, **kw)
 
     __call__ = searchResults
 
     def search(self, *args, **kw):
         # Wrap search() the same way that searchResults() is
-        query = {}
 
+        # Make sure any pending index tasks have been processed
+        processQueue()
+
+        query = {}
         if args:
             query = args[0]
         elif 'query_request' in kw:
@@ -405,7 +545,7 @@ class CatalogTool(PloneBaseTool, BaseTool):
         user = _getAuthenticatedUser(self)
         query['allowedRolesAndUsers'] = self._listAllowedRolesAndUsers(user)
 
-        if not _checkPermission(AccessInactivePortalContent, self):
+        if not self.allow_inactive(kw):
             query['effectiveRange'] = DateTime()
 
         kw['query_request'] = query
@@ -417,38 +557,16 @@ class CatalogTool(PloneBaseTool, BaseTool):
         # Empties catalog, then finds all contentish objects (i.e. objects
         # with an indexObject method), and reindexes them.
         # This may take a long time.
-
-        def indexObject(obj, path):
-            if (base_hasattr(obj, 'indexObject') and
-                    safe_callable(obj.indexObject)):
-                try:
-                    obj.indexObject()
-
-                    # index conversions from plone.app.discussion
-                    annotions = IAnnotations(obj)
-                    catalog = getToolByName(obj, "portal_catalog")
-                    if DISCUSSION_ANNOTATION_KEY in annotions:
-                        conversation = annotions[DISCUSSION_ANNOTATION_KEY]
-                        conversation = conversation.__of__(obj)
-                        for comment in conversation.getComments():
-                            try:
-                                if catalog:
-                                    catalog.indexObject(comment)
-                            except StopIteration:  # pragma: no cover
-                                pass
-
-
-                except TypeError:
-                    # Catalogs have 'indexObject' as well, but they
-                    # take different args, and will fail
-                    pass
         self.manage_catalogClear()
         portal = aq_parent(aq_inner(self))
+        idxs = list(self.indexes())
+        index_content = ContentIndexer(aq_inner(self), idxs=idxs)
         portal.ZopeFindAndApply(
             portal,
             search_sub=True,
-            apply_func=indexObject
+            apply_func=index_content,
         )
+        processQueue()
 
     @security.protected(ManageZCatalogEntries)
     def manage_catalogRebuild(self, RESPONSE=None, URL1=None):
@@ -457,15 +575,28 @@ class CatalogTool(PloneBaseTool, BaseTool):
         """
         elapse = time.time()
         c_elapse = time.clock()
+        num_before = len(self._catalog)
 
+        logger.info(
+            'Catalog Rebuilt started. '
+            '{0} objects cataloged before clear'.format(num_before)
+        )
         self.clearFindAndRebuild()
 
         elapse = time.time() - elapse
         c_elapse = time.clock() - c_elapse
 
-        msg = ('Catalog Rebuilt\n'
-               'Total time: %s\n'
-               'Total CPU time: %s' % (repr(elapse), repr(c_elapse)))
+        msg = (
+            'Catalog Rebuilt\n'
+            '    Total time: {0}\n'
+            '    Total CPU time: {1}\n'
+            '    Number of objects changed from {2} to {3}'.format(
+                repr(elapse),
+                repr(c_elapse),
+                num_before,
+                len(self._catalog),
+            )
+        )
         logger.info(msg)
 
         if RESPONSE is not None:
@@ -473,4 +604,5 @@ class CatalogTool(PloneBaseTool, BaseTool):
                 URL1 + '/manage_catalogAdvanced?manage_tabs_message=' +
                 urllib.quote(msg))
 
+
 InitializeClass(CatalogTool)
