@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from AccessControl import ClassSecurityInfo
 from App.class_init import InitializeClass
 from App.special_dtml import DTMLFile
@@ -14,9 +15,10 @@ from Products.CMFCore.utils import UniqueObject
 from Products.CMFPlone import PloneMessageFactory as _
 from Products.CMFPlone.interfaces import IControlPanel
 from Products.CMFPlone.PloneBaseTool import PloneBaseTool
+from zope.component.hooks import getSite
 from zope.i18n import translate
 from zope.i18nmessageid import Message
-from zope.interface import implements
+from zope.interface import implementer
 
 
 class PloneConfiglet(ActionInformation):
@@ -40,14 +42,13 @@ class PloneConfiglet(ActionInformation):
         return res
 
 
+@implementer(IControlPanel)
 class PloneControlPanel(PloneBaseTool, UniqueObject,
                         Folder, ActionProviderBase, PropertyManager):
     """Weave together the various sources of "actions" which
     are apropos to the current user and context.
     """
 
-    implements(IControlPanel)
-
     security = ClassSecurityInfo()
 
     id = 'portal_controlpanel'
@@ -194,7 +195,7 @@ class PloneControlPanel(PloneBaseTool, UniqueObject,
             except ValueError:
                 visible = 0
 
-        if not isinstance(permissions, basestring):
+        if isinstance(permissions, basestring):
             permissions = (permissions, )
 
         return PloneConfiglet(id=id,
@@ -295,5 +296,15 @@ class PloneControlPanel(PloneBaseTool, UniqueObject,
             manage_tabs_message=manage_tabs_message,
         )
 
+    @property
+    def site_url(self):
+        """Return the absolute URL to the current site, which is likely not
+        necessarily the portal root.
+        Used by ``portlet_prefs`` to construct the URL to
+        ``@@overview-controlpanel``.
+        """
+        return getSite().absolute_url()
+
+
 InitializeClass(PloneControlPanel)
 registerToolInterface('portal_controlpanel', IControlPanel)
