@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from plone.memoize import view
 from App.class_init import InitializeClass
 from zExceptions import NotFound
@@ -24,7 +25,7 @@ from Products.CMFCore.permissions import AccessContentsInformation, \
     ModifyPortalContent
 from Products.CMFPlone.DublinCore import DefaultDublinCoreImpl
 
-from zope.interface import implements
+from zope.interface import implementer
 
 
 class ReplaceableWrapper:
@@ -93,6 +94,7 @@ class OrderedContainer(Folder, OrderSupport):
 InitializeClass(OrderedContainer)
 
 
+@implementer(IWriteLock)
 class BasePloneFolder(CatalogAware, WorkflowAware, OpaqueItemManager,
                       PortalFolderBase, DefaultDublinCoreImpl):
     """Implements basic Plone folder functionality except ordering support.
@@ -100,8 +102,6 @@ class BasePloneFolder(CatalogAware, WorkflowAware, OpaqueItemManager,
 
     security = ClassSecurityInfo()
 
-    implements(IWriteLock)
-
     manage_options = Folder.manage_options + \
         WorkflowAware.manage_options
 
