@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from AccessControl import ClassSecurityInfo, Unauthorized
 from AccessControl import getSecurityManager
 from AccessControl.requestmethod import postonly
@@ -17,12 +18,13 @@ from Products.CMFPlone.interfaces import ISecuritySchema
 from Products.CMFPlone.permissions import ManagePortal
 from Products.CMFPlone.PloneBaseTool import PloneBaseTool
 from Products.CMFPlone.PloneTool import EMAIL_RE
+from Products.CMFPlone import PloneMessageFactory as _
 from Products.PluggableAuthService.interfaces.authservice import IPluggableAuthService
 from Products.PluggableAuthService.interfaces.plugins import IValidationPlugin
 from Products.PluggableAuthService.permissions import SetOwnPassword
 from smtplib import SMTPException, SMTPRecipientsRefused
 from zope.component import getUtility
-from zope.i18nmessageid import MessageFactory
+from zope.component import getMultiAdapter
 from zope.schema import ValidationError
 import random
 import re
@@ -32,8 +34,6 @@ import re
 # - remove vowels to avoid spelling words
 invalid_password_chars = ['a', 'e', 'i', 'o', 'u', 'y', 'l', 'q']
 
-_ = MessageFactory('plone')
-
 
 def getValidPasswordChars():
     password_chars = []
@@ -173,7 +173,7 @@ class RegistrationTool(PloneBaseTool, BaseTool):
         # o If the password is valid, return None.
         # o If not, return a string explaining why.
         err = self.pasValidation('password', password)
-        if err and (password == '' or not _checkPermission(ManagePortal, self)):
+        if err:
             return err
 
         if confirm is not None and confirm != password:
@@ -367,8 +367,10 @@ class RegistrationTool(PloneBaseTool, BaseTool):
 
         registry = getUtility(IRegistry)
         encoding = registry.get('plone.email_charset', 'utf-8')
-        mail_text = self.mail_password_template(
-            self, REQUEST, member=member, reset=reset,
+        mail_password_template = getMultiAdapter(
+            (self, self.REQUEST), name='mail_password_template')
+        mail_text = mail_password_template(
+            member=member, reset=reset,
             password=member.getPassword(), charset=encoding)
         # The mail headers are not properly encoded we need to extract
         # them and let MailHost manage the encoding.
@@ -390,12 +392,11 @@ class RegistrationTool(PloneBaseTool, BaseTool):
                 _(u'Recipient address rejected by server.'))
         except SMTPException as e:
             raise(e)
-        # return the rendered template "mail_password_response.pt"
-        # (in Products.PasswordResetTool)
-        return self.mail_password_response(self, REQUEST)
+        mail_password_response = getMultiAdapter(
+            (self, self.REQUEST), name='mail_password_response')
+        return mail_password_response()
 
     security.declarePublic('registeredNotify')
-
     def registeredNotify(self, new_member_id):
         # Wrapper around registeredNotify.
         membership = getToolByName(self, 'portal_membership')
@@ -421,9 +422,10 @@ class RegistrationTool(PloneBaseTool, BaseTool):
         # don't need to worry about 'UseMailHost' permissions).
         registry = getUtility(IRegistry)
         encoding = registry.get('plone.email_charset', 'utf-8')
-        mail_text = self.registered_notify_template(
-            self, self.REQUEST, member=member, reset=reset, email=email,
-            charset=encoding)
+        registered_notify_template = getMultiAdapter(
+            (self, self.REQUEST), name='registered_notify_template')
+        mail_text = registered_notify_template(
+            member=member, reset=reset, email=email, charset=encoding)
 
         # The mail headers are not properly encoded we need to extract
         # them and let MailHost manage the encoding.
@@ -438,7 +440,9 @@ class RegistrationTool(PloneBaseTool, BaseTool):
         host.send(mail_text, m_to, m_from, subject=subject, charset=encoding,
                   msg_type=msg_type, immediate=True)
 
-        return self.mail_password_response(self, self.REQUEST)
+        mail_password_response = getMultiAdapter(
+            (self, self.REQUEST), name='mail_password_response')
+        return mail_password_response()
 
     security.declareProtected(ManagePortal, 'editMember')
 
