@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from zope.component import getMultiAdapter
 
 from Products.CMFCore.utils import getToolByName
@@ -12,6 +13,16 @@ from Products.CMFCore.permissions import ManagePortal
 from Products.DCWorkflow.Transitions import TRIGGER_USER_ACTION
 from Products.CMFPlone.PloneBaseTool import PloneBaseTool
 
+import pkg_resources
+
+try:
+    pkg_resources.get_distribution('plone.app.multilingual')
+except pkg_resources.DistributionNotFound:
+    has_new_lang_bypass = False
+else:
+    has_new_lang_bypass = int(pkg_resources.get_distribution(
+        'plone.app.multilingual').version.split('.')[0]) > 1
+
 
 class WorkflowTool(PloneBaseTool, BaseTool):
 
@@ -230,7 +241,14 @@ class WorkflowTool(PloneBaseTool, BaseTool):
                     # Support LinguaPlone review situations, you want to see
                     # content in *all* languages
                     if 'Language' not in catalog_vars:
-                        catalog_vars['Language'] = 'all'
+                        if has_new_lang_bypass:
+                            catalog_vars['path'] = '/'
+                        else:
+                            catalog_vars['Language'] = 'all'
+                    # Include inactive content in result list. This is
+                    # especially important for content scheduled to go public
+                    # in the future, but needs to be reviewed before this.
+                    catalog_vars['show_inactive'] = True
                     for result in catalog.searchResults(catalog_vars):
                         o = result.getObject()
                         if o \
