@@ -1,19 +1,23 @@
-from zope.interface import implements
+# -*- coding: utf-8 -*-
+from zope.interface import implementer
+from zope.component import getMultiAdapter
 from zope.component.interfaces import ObjectEvent
-
 from Products.CMFCore.utils import getToolByName
+from Products.CMFPlone.utils import get_installer
 
 from interfaces import ISiteManagerCreatedEvent
 from interfaces import IReorderedEvent
 
 
+@implementer(ISiteManagerCreatedEvent)
 class SiteManagerCreatedEvent(ObjectEvent):
 
-    implements(ISiteManagerCreatedEvent)
+    pass
 
 
+@implementer(IReorderedEvent)
 class ReorderedEvent(ObjectEvent):
-    implements(IReorderedEvent)
+    pass
 
 
 def profileImportedEventHandler(event):
@@ -26,13 +30,10 @@ def profileImportedEventHandler(event):
         return
     profile_id = profile_id.replace('profile-', '')
     gs = event.tool
-    qi = getToolByName(gs, 'portal_quickinstaller', None)
-    if qi is None:
-        # CMF-only site, or a test run.
-        return
     installed_version = gs.getLastVersionForProfile(profile_id)
     if installed_version == (u'latest',):
-        actual_version = qi.getLatestUpgradeStep(profile_id)
+        qi = get_installer(gs, gs.REQUEST)
+        actual_version = qi.get_latest_upgrade_step(profile_id)
         gs.setLastVersionForProfile(profile_id, actual_version)
 
 
