@@ -28,8 +28,10 @@ require([
   'mockup-patterns-tinymce',
   'mockup-patterns-inlinevalidation',
   'mockup-patterns-structure',
+  'mockup-patterns-structureupdater',
   'mockup-patterns-recurrence',
-  'plone-patterns-portletmanager'
+  'plone-patterns-portletmanager',
+  'plone-patterns-toolbar',
 ], function() {
   'use strict';
 });
