@@ -1,6 +1,6 @@
+# -*- coding: utf-8 -*-
 from Products.CMFPlone.tests import PloneTestCase
 from Products.CMFPlone.tests import dummy
-from Products.CMFCore.tests.base.testcase import WarningInterceptor
 
 from zope.component import getGlobalSiteManager
 from zope.component import getSiteManager
@@ -19,6 +19,7 @@ from Products.CMFCore.permissions import AccessInactivePortalContent
 from Products.CMFCore.utils import getToolByName
 from Products.CMFPlone import setuphandlers
 from Products.CMFPlone.factory import _DEFAULT_PROFILE
+from Products.CMFPlone.interfaces import IFilterSchema
 from Products.CMFPlone.interfaces import INavigationSchema
 from Products.CMFPlone.interfaces import ISearchSchema
 from Products.CMFPlone.UnicodeSplitter import Splitter, I18NNormalizer
@@ -38,7 +39,7 @@ from plone.protect import createToken
 from plone.registry.interfaces import IRegistry
 
 
-class TestPortalCreation(PloneTestCase.PloneTestCase, WarningInterceptor):
+class TestPortalCreation(PloneTestCase.PloneTestCase):
 
     def afterSetUp(self):
         self.membership = self.portal.portal_membership
@@ -56,9 +57,6 @@ class TestPortalCreation(PloneTestCase.PloneTestCase, WarningInterceptor):
         self.javascripts = self.portal.portal_javascripts
         self.setup = self.portal.portal_setup
 
-    def beforeTearDown(self):
-        self._free_warning_output()
-
     def testInstanceVersion(self):
         # Test if the version of the instance has been set
         mt = getToolByName(self.portal, 'portal_migration')
@@ -189,7 +187,7 @@ class TestPortalCreation(PloneTestCase.PloneTestCase, WarningInterceptor):
         self.assertFalse(
             self.properties.navtree_properties.hasProperty('showAllParents'))
         self.assertFalse(
-            self.properties.navtree_properties.hasProperty('metaTypesNotToList'))
+            self.properties.navtree_properties.hasProperty('metaTypesNotToList'))  # noqa
         self.assertFalse(
             self.properties.navtree_properties.hasProperty('sortAttribute'))
         self.assertFalse(
@@ -225,6 +223,13 @@ class TestPortalCreation(PloneTestCase.PloneTestCase, WarningInterceptor):
         self.assertTrue('plone.types_not_searched' in registry)
         self.assertTrue('Plone Site' in settings.types_not_searched)
 
+    def testDefaultSortOrderProperty(self):
+        # We should have an sort_on property
+        registry = getUtility(IRegistry)
+        settings = registry.forInterface(ISearchSchema, prefix="plone")
+        self.assertIn('plone.sort_on', registry)
+        self.assertEqual(settings.sort_on, 'relevance')
+
     def testNonDefaultPageTypes(self):
         # We should have a default_page_types setting
         registry = self.portal.portal_registry
@@ -352,7 +357,6 @@ class TestPortalCreation(PloneTestCase.PloneTestCase, WarningInterceptor):
                 self.assertTrue(a.visible)
 
     def testDefaultGroupsAdded(self):
-        self._trap_warning_output()
         self.assertTrue('Administrators' in self.groups.listGroupIds())
         self.assertTrue('Reviewers' in self.groups.listGroupIds())
 
@@ -555,7 +559,7 @@ class TestPortalCreation(PloneTestCase.PloneTestCase, WarningInterceptor):
         urls = [a['url'] for a in buttons]
         for url in urls:
             self.assertFalse('index_html' not in url,
-                             'Action wrongly applied to parent object %s' % url)
+                             'Action wrongly applied to parent object %s' % url)  # noqa
 
     def testObjectButtonActionsPerformCorrectAction(self):
         # only a manager would have proper permissions
@@ -614,10 +618,10 @@ class TestPortalCreation(PloneTestCase.PloneTestCase, WarningInterceptor):
         self.assertTrue(member_has_permission['selected'])
 
     def testDiscussionItemWorkflow(self):
-        # By default the discussion item has the one_state_workflow
+        # By default the discussion item has the comment_one_state_workflow
         self.assertEqual(
             self.workflow.getChainForPortalType('Discussion Item'),
-            ('one_state_workflow',))
+            ('comment_one_state_workflow',))
 
     def testFolderHasFolderListingView(self):
         # Folder type should allow 'folder_listing'
@@ -638,11 +642,14 @@ class TestPortalCreation(PloneTestCase.PloneTestCase, WarningInterceptor):
         self.assertTrue('atct_album_view' in self.types.Folder.view_methods)
 
     def testConfigurableSafeHtmlTransform(self):
+        registry = getUtility(IRegistry)
+        settings = registry.forInterface(
+            IFilterSchema, prefix="plone")
         # The safe_html transformation should be configurable
         try:
-            self.transforms.safe_html.get_parameter_value('disable_transform')
+            settings.disable_filtering
         except (AttributeError, KeyError):
-            self.fail('safe_html transformation not updated')
+            self.fail('Disabling of safe_html should be possible!')
 
     def testvcXMLRPCRemoved(self):
         # vcXMLRPC.js should no longer be registered
@@ -901,16 +908,8 @@ class TestPortalBugs(PloneTestCase.PloneTestCase):
         self.loginAsPortalOwner()
         portal = self.portal
         portal.manage_delObjects(['Members'])
-
-        class FakeContext:
-
-            def getSite(self):
-                return portal
-
-            def readDataFile(self, filename):
-                return True  # Anything other than None runs the step
-
-        setuphandlers.importFinalSteps(FakeContext())  # raises error if fail
+        setup_tool = getToolByName(self.portal, 'portal_setup')
+        setuphandlers.importFinalSteps(setup_tool)  # raises error if fail
         self.assertTrue(1 == 1)
 
 
